@@ -1,9 +1,13 @@
 use crate::render::portable::DatasetError;
+use crate::spec::ConfigError::{
+    ConflictingConfiguration, PlotAndTablePresentConfiguration, WrongScaleType,
+};
 use anyhow::Result;
 use anyhow::{bail, Context};
 use derefable::Derefable;
 use itertools::Itertools;
 use lazy_static::lazy_static;
+use log::warn;
 use regex::Regex;
 use serde::Deserialize;
 use serde::Serialize;
@@ -45,6 +49,76 @@ impl ItemsSpec {
         }
         Ok(items_spec)
     }
+
+    pub(crate) fn validate(&self) -> Result<()> {
+        for (name, view) in &self.views {
+            if self.datasets.get(&view.dataset).is_none() {
+                bail!(ConfigError::MissingDataset {
+                    dataset: view.dataset.to_string()
+                })
+            }
+            if let Some(render_table) = &view.render_table {
+                if !render_table.is_empty() && view.render_plot.is_some() {
+                    bail!(PlotAndTablePresentConfiguration {
+                        view: name.to_string()
+                    });
+                }
+                let dataset = self.datasets.get(&view.dataset).unwrap();
+                let mut reader = csv::ReaderBuilder::new()
+                    .delimiter(dataset.separator as u8)
+                    .from_path(&dataset.path)?;
+                let titles = reader.headers()?.iter().map(|s| s.to_owned()).collect_vec();
+                for (column, render_columns) in render_table {
+                    if !titles.contains(column) && !render_columns.optional {
+                        warn!("Found render-table definition for column {} that is not part of the given dataset.", &column);
+                    }
+                    let mut possible_conflicting = Vec::new();
+                    if render_columns.ellipsis.is_some() {
+                        possible_conflicting.push("ellipsis".to_string());
+                    }
+                    if render_columns.link_to_url.is_some() {
+                        possible_conflicting.push("link-to-url".to_string());
+                    }
+                    if render_columns.custom.is_some() {
+                        possible_conflicting.push("custom".to_string());
+                    }
+                    if render_columns.custom_plot.is_some() {
+                        possible_conflicting.push("custom-plot".to_string());
+                    }
+                    if render_columns.plot.is_some() {
+                        possible_conflicting.push("plot".to_string());
+                    }
+                    if possible_conflicting.len() > 1 {
+                        bail!(ConflictingConfiguration {
+                            view: name.to_string(),
+                            column: column.to_string(),
+                            conflict: possible_conflicting
+                        })
+                    }
+
+                    if let Some(plot) = &render_columns.plot {
+                        if let Some(heatmap) = &plot.heatmap {
+                            if !SCALE_TYPES.contains(&&*heatmap.scale_type) {
+                                bail!(WrongScaleType {
+                                    scale_type: heatmap.scale_type.clone(),
+                                    possible_scale_types: SCALE_TYPES.to_vec(),
+                                })
+                            }
+                        }
+                        if let Some(tick_plot) = &plot.tick_plot {
+                            if !SCALE_TYPES.contains(&&*tick_plot.scale_type) {
+                                bail!(WrongScaleType {
+                                    scale_type: tick_plot.scale_type.clone(),
+                                    possible_scale_types: SCALE_TYPES.to_vec(),
+                                })
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        Ok(())
+    }
 }
 
 fn default_separator() -> char {
@@ -82,6 +156,8 @@ pub(crate) struct DatasetSpecs {
 #[derive(Deserialize, Debug, Clone, PartialEq)]
 #[serde(rename_all(deserialize = "kebab-case"))]
 pub(crate) struct ItemSpecs {
+    #[serde(default)]
+    pub(crate) hidden: bool,
     pub(crate) dataset: String,
     #[serde(default = "default_page_size")]
     pub(crate) page_size: usize,
@@ -197,6 +273,8 @@ pub(crate) struct RenderColumnSpec {
     pub(crate) plot: Option<PlotSpec>,
     #[serde(default)]
     pub(crate) custom_plot: Option<CustomPlot>,
+    #[serde(default)]
+    pub(crate) ellipsis: Option<u32>,
 }
 
 #[derive(Deserialize, Debug, Clone, PartialEq)]
@@ -290,6 +368,23 @@ pub(crate) struct Heatmap {
     pub(crate) domain: Option<Vec<String>>,
 }
 
+static SCALE_TYPES: [&'static str; 14] = [
+    "linear",
+    "pow",
+    "sqrt",
+    "symlog",
+    "log",
+    "time",
+    "utc",
+    "ordinal",
+    "band",
+    "point",
+    "bin-ordinal",
+    "quantile",
+    "quantize",
+    "threshold",
+];
+
 #[derive(Error, Debug)]
 pub enum ConfigError {
     #[error("Could not find column with index {index:?} under path {table_path:?} with only {header_length:?} columns.")]
@@ -300,6 +395,21 @@ pub enum ConfigError {
     },
     #[error("Column {column:?} under path {table_path:?} seems to have multiple definitions. Please check your config file.")]
     DuplicateColumn { column: String, table_path: PathBuf },
+    #[error("Could not find dataset named {dataset:?} in given config.")]
+    MissingDataset { dataset: String },
+    #[error("View {view:?} consists of a configuration with render-plot and render-table present while only one should be present. If you want both please define two separate views.")]
+    PlotAndTablePresentConfiguration { view: String },
+    #[error("Found conflicting render-table configuration for column {column:?} of view {view:?}. The conflicting configuration are {conflict:?}.")]
+    ConflictingConfiguration {
+        view: String,
+        column: String,
+        conflict: Vec<String>,
+    },
+    #[error("Given scale type {scale_type:?} is not valid, please choose one of the following scale types: {possible_scale_types:?}.")]
+    WrongScaleType {
+        scale_type: String,
+        possible_scale_types: Vec<&'static str>,
+    },
 }
 
 #[cfg(test)]
@@ -320,6 +430,7 @@ mod tests {
             link_to_url: Some(String::from("https://www.rust-lang.org")),
             plot: None,
             custom_plot: None,
+            ellipsis: None,
         };
 
         let expected_dataset_spec = DatasetSpecs {
@@ -330,6 +441,7 @@ mod tests {
         };
 
         let expected_table_spec = ItemSpecs {
+            hidden: false,
             dataset: "table-a".to_string(),
             page_size: 100,
             pin_columns: 1,
@@ -393,6 +505,7 @@ mod tests {
         };
 
         let expected_item_spec = ItemSpecs {
+            hidden: false,
             dataset: "table-a".to_string(),
             page_size: 100,
             pin_columns: 0,
@@ -409,27 +522,105 @@ mod tests {
         };
 
         let raw_config = r#"
-    datasets:
-        table-a:
-            path: test.tsv
-            links:
-                my-link:
-                    column: test
-                    view: other-table
-    default-view: table-a
-    views:
-        plot-a:
-            dataset: table-a
-            desc: "my table"
-            render-plot:
-                spec: |
-                    {'$schema': 'https://vega.github.io/schema/vega-lite/v5.json'}
-    "#;
+            datasets:
+                table-a:
+                    path: test.tsv
+                    links:
+                        my-link:
+                            column: test
+                            view: other-table
+            default-view: table-a
+            views:
+                plot-a:
+                    dataset: table-a
+                    desc: "my table"
+                    render-plot:
+                        spec: |
+                            {'$schema': 'https://vega.github.io/schema/vega-lite/v5.json'}
+            "#;
 
         let config: ItemsSpec = serde_yaml::from_str(raw_config).unwrap();
         assert_eq!(config, expected_config);
     }
 
+    #[test]
+    fn test_valid_config_validation() {
+        let config = ItemsSpec::from_file(".examples/example-config.yaml").unwrap();
+        assert!(config.validate().is_ok());
+    }
+
+    #[test]
+    fn test_missing_dataset_config_validation() {
+        let raw_config = r#"
+            datasets:
+                table-a:
+                    path: test.tsv
+            views:
+                plot-b:
+                    dataset: table-b
+            "#;
+        let config: ItemsSpec = serde_yaml::from_str(raw_config).unwrap();
+        assert!(config.validate().is_err());
+    }
+
+    #[test]
+    fn test_conflicting_config_validation() {
+        let raw_config = r#"
+            datasets:
+                table-a:
+                    path: data.csv
+            views:
+                table-a:
+                    dataset: table-a
+                    render-table:
+                        some-column:
+                            plot:
+                                ticks:
+                                    scale: linear
+                            ellipsis: 25
+            "#;
+        let config: ItemsSpec = serde_yaml::from_str(raw_config).unwrap();
+        assert!(config.validate().is_err());
+    }
+
+    #[test]
+    fn test_wrong_scale_config_validation() {
+        let raw_config = r#"
+            datasets:
+                table-a:
+                    path: data.csv
+            views:
+                table-a:
+                    dataset: table-a
+                    render-table:
+                        some-column:
+                            plot:
+                                heatmap:
+                                    scale: inverse-quadruplic
+            "#;
+        let config: ItemsSpec = serde_yaml::from_str(raw_config).unwrap();
+        assert!(config.validate().is_err());
+    }
+
+    #[test]
+    fn test_plot_and_table_present_config_validation() {
+        let raw_config = r#"
+            datasets:
+                table-a:
+                    path: data.csv
+            views:
+                table-a:
+                    dataset: table-a
+                    render-table:
+                        x:
+                            link-to-url: "https://lmgtfy.app/?q=Is {name} in {movie}?"
+                    render-plot:
+                        spec-path: ".examples/specs/movies.vl.json"
+            "#;
+        let config: ItemsSpec = serde_yaml::from_str(raw_config).unwrap();
+        assert!(config.validate().is_err());
+    }
+
     #[test]
     fn test_config_preprocessing() {
         let config = ItemsSpec::from_file(".examples/example-config.yaml").unwrap();
@@ -447,6 +638,7 @@ mod tests {
             link_to_url: None,
             plot: None,
             custom_plot: None,
+            ellipsis: None,
         };
         assert_eq!(
             oscar_config.get("oscar_no").unwrap().to_owned(),
